/*
 * wave.c / mkwave.c
 *
 * (c) Lennart Steinke (steinke@adv-boeblingen.de)
 * http://mars.adv-boeblingen.de/~steinke
 *
 *
 * Creates an realtime effect similar to rain drops hitting
 * a water surface
 *
 * If you're using this code, please give me some credit and
 * drop me a mail.
 *
 *
 * mkwave.c
 *			creates the filter buffers and saves them to disk,
 *          then displays the fx
 *
 * wave.c
 *			load the buffers and displays the fx
 *
 */

#include <jlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define WAIT fgetc(stdin);
#define RAD(x)  (((double)(x))/180.0*PI)
#define W_WIDTH 96
#define W_WIDTH_HALF 48
#define W_FACTOR (320 / W_WIDTH_HALF)
#define DROP_COUNT 6

double sin_table[360],
	   r_sqrt_table[3600];

int height_table[32][320];

typedef struct
		{
			SHORT x,y,t;
		}
		rain;

int f,s,q, to_go = DROP_COUNT;
buffer_rec	*offscreen,
			*tmp,
			*wave[32],
			*grad[32];

rain    rain_drop[DROP_COUNT];

void init(void);
void done(void);
void fatal_error(char *s);


void main(int argc, char **argv)
{
	int a,x,sx,y,t,min=0xffff,max=-0xffff;
	image *pic;
	double s;

	f=40;
	s=10;
	q=100;

	init();




	for (sx=1; sx < 320; sx+=s)
	{
		for (a=sx,x=sx; x>0; x--,a-=4)
		{
			if (a<0)
				a+=360;
			height_table[sx/10][x]=sin_table[a%360]*r_sqrt_table[sx+x]*(double)x/sx;
			if ((a % 180) < 90)
			{
				height_table[sx/10][x-1]=sin_table[a%360]*r_sqrt_table[sx+x]*(double)x/sx;
				x--;
			}
		}
		for (a=sx, x=sx; a<1200; a+=4,x++)
		{

			height_table[sx/10][x]=sin_table[a%360]*r_sqrt_table[sx+x];
			if ((a % 180) < 90)
			{
				height_table[sx/10][x+1]=sin_table[a%360]*r_sqrt_table[sx+x];
				x++;
			}
		}
	}

	/*
	 * Calc the water surface
	 *
	 * height = height_table[t][(int) (x / cos(atan2(y,x)))]
	 */


	for (t=0; t<32; t++)
	{
		wave[t]=buff_init(W_WIDTH,W_WIDTH);
		grad[t]=buff_init(W_WIDTH,W_WIDTH);
		for (x=0; x<W_WIDTH;x++)
		{
			for (y=0; y<W_WIDTH; y++)
			{
				a=sqrt( (y-W_WIDTH_HALF)*(y-W_WIDTH_HALF) +
						(x-W_WIDTH_HALF) * (x-W_WIDTH_HALF))*W_FACTOR;

				if (!(a<0 || a > 320))
				{
					buff_draw_point(wave[t],x,y,(UBYTE)height_table[t][a]);
				}
				else
					buff_draw_point(wave[t], x,y,0);
			}
		}


#define BGP(x,y) ((BYTE)buff_get_point(wave[t],x,y))
		for (x=0; x<W_WIDTH;x++)
		{
			for (y=0; y<W_WIDTH;y++)
			{
				buff_draw_point(
					grad[t],x,y,
						a=(UBYTE)
						( BGP(x-1,y-1)+ BGP(x,y-1)+ BGP(x+1,y-1)-
						  BGP(x-1,y+1)- BGP(x,y+1)- BGP(x+1,y+1)));
			}
		}
	}

	{
		FILE *f;
		f=fopen("wave.dat","wb");
		if (f)
		{
			for (a=0; a<32; a++)
			{
				fwrite(B_BUFF_PTR(grad[a]),sizeof(UBYTE),W_WIDTH*W_WIDTH,f);
			}

			fclose(f);
		}
	}


	pic=image_load_pcx("back.pcx");
	if (!pic)
		fatal_error("Image load failed.");
	screen_block_set_pal(IMG_PALETTE(pic));
	buff_blit_img_to(offscreen,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic)-1);
    tmp = buff_init(SCREEN_WIDTH,SCREEN_HEIGHT);
	if (tmp)
		buff_blit_img_to(tmp,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic));
	else
		fatal_error("!");

	screen_blit_fs_buffer(offscreen);

	for (a=0; a< DROP_COUNT; a++)
	{
		rain_drop[a].x = (rand() % (SCREEN_WIDTH -  W_WIDTH- 10)) + W_WIDTH_HALF;
		rain_drop[a].y = (rand() % (SCREEN_HEIGHT - W_WIDTH- 10)) + W_WIDTH_HALF;
		rain_drop[a].t = rand() % 32 -16;
	}

	WAIT;

	to_go=DROP_COUNT;
	while(to_go)
	{
		for (a=0; a<DROP_COUNT; a++)
		{
			if (rain_drop[a].t<0)
			{
				rain_drop[a].t++;
				continue;
			}
			else if (rain_drop[a].t ==32)
			{
				to_go--;
				rain_drop[a].t ++;
				continue;
			}
			else if (rain_drop[a].t > 32)
				continue;

			t=rain_drop[a].t;
			for (x=0; x<W_WIDTH; x++)
			{
				for (y=0; y<W_WIDTH;y++)
				{
					buff_draw_point(
						offscreen,
						x+rain_drop[a].x,y+rain_drop[a].y,
						buff_get_point(tmp,x+rain_drop[a].x, y+rain_drop[a].y+(BYTE) buff_get_point(grad[t],x,y)));
				}
			}
			rain_drop[a].t++;
		}
        screen_blit_fs_buffer(offscreen);
		buff_blit_img_to(offscreen,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic));
	}
	screen_blit_fs_buffer(offscreen);
	WAIT;



	done();
}

void
init(void)
{
	int angle;
	UBYTE *pal;

	/* Build the sin table */

	for (angle=0; angle<360; angle++)
	{
		sin_table [angle] = sin(RAD(angle));
	}
	r_sqrt_table[0]=0;
	for (angle=1; angle<3200; angle++)
	{
		/*
		if (angle<271)
			r_sqrt_table[angle] =f*(1-(angle/270.0))*10;
		else
			r_sqrt_table[angle] =0;
		*/
		r_sqrt_table[angle]=exp(-angle/(float)q)*f;
	}



    /* Create an offscreen buffer with the size of the screen */
	offscreen = buff_init(SCREEN_WIDTH,SCREEN_HEIGHT);
	

	if (!offscreen)
		fatal_error("Offscreen buffer init failed");

	/* Switch to graph mode, and load an palette */
	screen_set_video_mode();

	pal=pal_load("blue.pal");
	screen_block_set_pal(pal);
}

void done(void)
{
	buff_free(offscreen);
	screen_restore_video_mode();
}

void fatal_error(char *s){

	screen_restore_video_mode();

	kb_closedown();


	printf("\nFatal error:%s\n\n",s);
	fflush(stdout);
	exit(0);
}
