/*
 * wave.c / mkwave.c
 *
 * (c) Lennart Steinke (steinke@adv-boeblingen.de)
 * http://mars.adv-boeblingen.de/~steinke
 *
 *
 * Creates an realtime effect similar to rain drops hitting
 * a water surface
 *
 * If you're using this code, please give me some credit and
 * drop me a mail.
 *
 *
 * mkwave.c
 *			creates the filter buffers and saves them to disk,
 *          then displays the fx
 *
 * wave.c
 *			load the buffers and displays the fx
 *
 */

#include <jlib.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#define WAIT fgetc(stdin)

/* Width of the filter buffer */
#define W_WIDTH 96
#define W_WIDTH_HALF 48
#define DROP_COUNT 6

typedef struct
		{
			SHORT x,y,t;
		}
		rain;

int to_go = DROP_COUNT;
buffer_rec	*offscreen,
			*tmp,
			*grad[32];

rain    rain_drop[DROP_COUNT];

void init(void);
void done(void);
void fatal_error(char *s);


void main(int argc, char **argv)
{
	int a,x,y,t;
	int x_,y_;

	
	image *pic;
    FILE *f;

	init();

		
	f=fopen("wave.dat","rb");
	if (f)
	{
		for (a=0; a<32; a++)
		{
			fread(B_BUFF_PTR(grad[a]),sizeof(UBYTE),W_WIDTH*W_WIDTH,f);
		}

		fclose(f);
	}
	else
	{
		fatal_error("Unble to load filter (wave.dat)");
	}


	/* Load the background image */
	pic=image_load_pcx("back.pcx");
	if (!pic)
		fatal_error("Image load failed.");

	screen_block_set_pal(IMG_PALETTE(pic));
	buff_blit_img_to(offscreen,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic)-1);


	/* Create a tmp buffer, and blit the pic to it */
	tmp = buff_init(SCREEN_WIDTH,SCREEN_HEIGHT);
	if (tmp)
		buff_blit_img_to(tmp,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic));
	else
		fatal_error("Tmp buffer couldn't be created");

	screen_blit_fs_buffer(offscreen);

	/* Create the "rain drops" */
	for (a=0; a< DROP_COUNT; a++)
	{
		rain_drop[a].x = (rand() % (SCREEN_WIDTH -  W_WIDTH- 10)) + W_WIDTH_HALF;
		rain_drop[a].y = (rand() % (SCREEN_HEIGHT - W_WIDTH- 10)) + W_WIDTH_HALF;
		rain_drop[a].t = rand() % 32 -16;
	}


	to_go=DROP_COUNT;
	while(to_go)
	{
		for (a=0; a<DROP_COUNT; a++)
		{
			if (rain_drop[a].t<0)
			{
				/* Drop hits water at t=0
				 * Nothing to do here
				 */
				rain_drop[a].t++;
				continue;
			}
			else if (rain_drop[a].t ==32)
			{
				/* Animation finished */
				to_go--;
				rain_drop[a].t ++;
				continue;
			}
			else if (rain_drop[a].t > 32)
				continue;

			t=rain_drop[a].t;

			x_=rain_drop[a].x;
			for (x=0; x<W_WIDTH; x++,x_++)
			{
				y_=rain_drop[a].y;
				for (y=0; y<W_WIDTH;y++,y_++)
				{
					/* This is REALLY slow
					 * Using a ptr to the buffer data would be a lot
					 * faster, but this code worked to test the fx,
					 * and this is all I wanted...
					 */
					buff_draw_point(
						offscreen,
						x_,y_,
						buff_get_point(tmp,x_,y_+(BYTE) buff_get_point(grad[t],x,y)));
				}
			}
			rain_drop[a].t++;
		}
        screen_blit_fs_buffer(offscreen);
		buff_blit_img_to(offscreen,0,0,pic,0,0,IMG_MAX_X(pic),IMG_MAX_Y(pic));
	}
	screen_blit_fs_buffer(offscreen);
	WAIT;

	done();
}

void
init(void)
{
	/* Create the filter buffers
	 * The wave uses the formula
	 * 		height= sin(alpha) * exp(-angle/100.0)*40;
	 */

	 int a;

	 for (a=0; a<32;a++)
		if (! (grad[a]=buff_init(W_WIDTH,W_WIDTH)) )
		{
			fatal_error("Filter buffer could not be created!");
		}



    /* Create an offscreen buffer with the size of the screen */
	offscreen = buff_init(SCREEN_WIDTH,SCREEN_HEIGHT);
	

	if (!offscreen)
		fatal_error("Offscreen buffer init failed");

	/* Switch to graph mode */
	screen_set_video_mode();
}

void done(void)
{
	buff_free(offscreen);
	screen_restore_video_mode();
}

void fatal_error(char *s){

	screen_restore_video_mode();

	kb_closedown();


	printf("\nFatal error:%s\n\n",s);
	fflush(stdout);
	exit(0);
}
